/* kbtst.c -- test program for the libkb keyboard library
 * Copyright (C) 1995, 1996 Markus F.X.J. Oberhumer
 * For conditions of distribution and use, see copyright notice in kb.h 
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <signal.h>

#include <kb.h>
#include "_kb.h"			/* _kb_port_debug(), _kb_error_debug() */
#include "_kbintro.h"		/* _kb_intro_text() */
#include "_kbconio.h"

#if !defined(CLK_TCK) && defined(CLOCKS_PER_SEC)
#  define CLK_TCK			CLOCKS_PER_SEC 
#endif


/*************************************************************************
// signal handlers
**************************************************************************/

static void sigInt( int dummyValue )
{
	printf("SIGINT received.\n");
	fflush(stdout);
	exit(1);
}


#if defined(SIGBREAK)
static void sigBreak( int dummyValue )
{
	printf("SIGBREAK received.\n");
	fflush(stdout);
	exit(2);
}
#endif


/***********************************************************************
// 
************************************************************************/

static void show_unexpected_keys(void)
{
	if (!_kb_error_debug_avail())
		return;

	printf("\n****** you found a bug ******\n\n");
	printf("list of unexpected codes: ");
	_kb_error_debug();
	printf("\n\n");
	printf("please write down these codes and send email to me, thanks\n");
	printf("Markus F.X.J. Oberhumer <markus.oberhumer@jk.uni-linz.ac.at>\n");
	fflush(stdout);
}


/***********************************************************************
// 
************************************************************************/

static int mode;

static int KBHIT(void)
{
	if (mode <= 2)
		return kb_kbhit();
	else if (mode == 3)
		return kbhit();
	else if (mode == 4)
		return kb_bios_kbhit();
	else
		return 0;
}


static unsigned long GETCH(void)
{
	if (mode == 1)
		return kb_inkey();
	else if (mode == 2)
		return kb_getch();
	else if (mode == 3)
		return getch();
	else if (mode == 4)
		return kb_bios_getch();
	else
		return 0;
}


static int esc_pressed(void)
{
	if (kb_key(KB_SCAN_ESC))
	{
		if ((kb_key(KB_SCAN_LCONTROL) || kb_key(KB_SCAN_RCONTROL)) &&
			(kb_key(KB_SCAN_LSHIFT) || kb_key(KB_SCAN_RSHIFT)))
			return 1;
		if (kb_key(KB_SCAN_ALTGR))
			return 1;
	}
	return 0;
}


/***********************************************************************
// graphical interface
************************************************************************/

#define cputtext(x,y,text)	gotoxy(x,y), cputs(text)

static int graphical_interface(void)
{
	int i, x, y;
	char m[255];
	unsigned k;

#if defined(__TURBOC__) || defined(__GO32__)
	struct text_info ti;
	gettextinfo(&ti);
#if 0
	if (ti.screenwidth < 80 || ti.screenheight < 25)
		return 0;
#endif
#endif
	
	if (!kb_install(0))
		return 0;
	atexit(show_unexpected_keys);

#if defined(__EMX__)
	v_init(); v_hidecursor();
#endif
	_setcursortype(_NOCURSOR);
	clrscr();

/* write intro */
	x = 1; y = 1;
	textcolor(YELLOW);
	cputtext(x,y++,_kb_intro_text(m));
	cputtext(x,y++,"libkb -- a free, advanced and portable keyboard library");
	textcolor(LIGHTGRAY);
	y++;
	cputtext(x,y++,"Please send your bug reports, fixes, enhancements or suggestions to:\n");
	cputtext(x,y++,"Markus F.X.J. Oberhumer <markus.oberhumer@jk.uni-linz.ac.at>\n");
	y++;
	cputtext(x,y++,"Keyboard interrupt installed.");
	cputtext(x,y++,"Hit any key to test, Ctrl-Shift-Esc or AltGr-Esc to quit");

/* write constant text */
	y += 2;
	x = 12;
	cputtext(x,y++,"          1         2         3         ");
	cputtext(x,y++,"0123456789012345678901234567890123456789");
	y += 2;
	cputtext(x,y++,"4         5         6         7         ");
	cputtext(x,y++,"0123456789012345678901234567890123456789");
	y += 2;
	cputtext(x,y++,"                    1         1");
	cputtext(x,y++,"8         9         0         1         ");
	cputtext(x,y++,"0123456789012345678901234567890123456789");
	cputtext(x+46,y-1,"shift flags");

	while (!esc_pressed())
	{
		/* print keys */
		i = 0;
		m[40] = 0;
		textcolor(GREEN);
		while (i < 40)
			m[i++] = kb_key(i) ? 0x1e : ' ';
		cputtext(x,y-9,m);
		while (i < 80)
			m[-40 + i++] = kb_key(i) ? 0x1e : ' ';
		cputtext(x,y-5,m);
		while (i < 120)
			m[-80 + i++] = kb_key(i) ? 0x1e : ' ';
		cputtext(x,y,m);

		/* print shift flags */
		k = kb_shift() & KB_SHIFT_ANY;
		for (i = 0; i < 12; i++, k >>= 1)
			m[i] = k & 1 ? 0x1e : ' ';
		m[i] = 0;
		textcolor(LIGHTRED);
		cputtext(x+46,y,m);

		/* print scan codes */
		sprintf(m,"%2d keys now pressed: ", kb_keys_pressed());
		for (i = 0; i < 128; i++)
			if (kb_key(i))
				sprintf(m+strlen(m), "%d ", i);
		for (i = strlen(m); i < 80; i++)
			m[i] = ' ';
		m[80] = 0;
		textcolor(LIGHTGRAY);
		cputtext(1,y+2,m);
	}

#if defined(__TURBOC__) || defined(__GO32__)
	textattr(ti.attribute);
#endif
	gotoxy(1,25);
	_setcursortype(_NORMALCURSOR);
	return 1;
}


/***********************************************************************
// show a keypress
************************************************************************/

static void show_keys(unsigned long c, unsigned long c2)
{
	long a = c & 0xff;

	printf("num=%3d  st=%04x  ", kb_keys_pressed(), kb_shift() & KB_SHIFT_ANY);

	if (a >= 32 && a < 127)
		printf("'%c'", (char) (a & 0x7f));
	else
		printf("   ");

	printf("  key=%08lx", c);

	/* Extended character ? */
	if (c2)
		printf(" %08lx", c2);

	printf("  ");
	_kb_port_debug();
	printf("\n");
	fflush(stdout);
}


/***********************************************************************
// 
************************************************************************/

static void speed_test(void)
{
	clock_t t1, t2;
	long i, n, k;

	printf("Running speed test (this can take a while)...\n");
	i = n = 100000L;
	k = 0;
	t1 = clock();
	do {
		if (KBHIT())
		{
			GETCH();
			k++;
		}
	} while (--i > 0);
	t2 = clock();

	printf("%ld keys were hit (%ld, %ld millisecs)\n",
		k, n, ((t2-t1)*1000L)/(long)(CLK_TCK));
}


/***********************************************************************
// 
************************************************************************/

int main(int argc, char *argv[])
{
	unsigned long c1, c2;
	int i;
	char s[80+1];

	signal(SIGINT, sigInt );
#if defined(SIGBREAK)
	signal(SIGBREAK, sigBreak );
#endif

	fputs("\n",stdout);
	fputs(_kb_intro_text(s),stdout);
	fputs("\n\n",stdout);
	printf("Please send your bug reports, fixes, enhancements or suggestions to:\n");
	printf("Markus F.X.J. Oberhumer <markus.oberhumer@jk.uni-linz.ac.at>\n");
	printf("\n");

	mode = 5;
	if (argc > 1)
		mode = atoi(argv[1]);
	if (mode < 1 || mode > 5)
		mode = 1;

	if (mode == 5)
	{
		if (graphical_interface())
			exit(0);
		else
			mode = 1;
	}
	if (mode <= 2)
	{
		if (kb_install(KB_FLAG_INIT_SHIFT_TOGGLES))
			atexit(show_unexpected_keys);
		else
			mode = 3;
	}
	if (mode <= 2)
	{
		printf("Keyboard interrupt installed.\n");
		printf("Hit any key to test, Ctrl-Shift-Esc or AltGr-Esc to quit\n\n");
	}
	else if (mode == 3)
	{
		printf("Using getch().\n");
		printf("Hit any key to test, Ctrl-Break to quit\n\n");
	}
	else if (mode == 4)
	{
		printf("Using kb_bios_getch().\n");
		printf("Hit any key to test, Ctrl-Break to quit\n\n");
	}


	if (argc > 2)
	{
		speed_test();
		exit(0);
	}


	while (!esc_pressed())
	{
		while (!KBHIT())
		{
			if (kb_shift() & KB_SHIFT_CONTROL_BREAK)
			{
				printf("Control-Break pressed\n");
				show_keys(0,0);
				_kb_shift &= ~(KB_SHIFT_CONTROL_BREAK);
			}
			if (kb_shift() & KB_SHIFT_PAUSE)
			{
				if (KB_ALL_MASK(kb_shift(), KB_SHIFT_ANY_CONTROL))
					printf("LControl-RControl-Break pressed\n");
				else
					printf("Pause pressed\n");
				show_keys(0,0);
				_kb_shift &= ~(KB_SHIFT_PAUSE);
			}
		}

		c1 = GETCH();
		c2 = c1 ? 0 : GETCH();
		show_keys(c1,c2);
	}

	printf("\n");
	printf("%d keys now pressed: ", kb_keys_pressed());
	for (i = 0; i < 128; i++)
		if (kb_key(i))
			printf("%02x ", i);
	printf("\n");
	fflush(stdout);

	show_unexpected_keys();

	return 0;
}


/*
vi:ts=4
*/
